@@ -4,8 +4,6 @@
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
-#include "solidhwtest.h"
-
 #include <QSignalSpy>
 #include <QTest>
 
@@ -27,12 +25,36 @@
 #error "FAKE_COMPUTER_XML not set. An XML file describing a computer is required for this test"
 #endif
 
-QTEST_MAIN(SolidHwTest)
+class SolidHwTest : public QObject
+{
+    Q_OBJECT
+private Q_SLOTS:
+    void initTestCase();
+    void testAllDevices();
+    void testDeviceBasicFeatures();
+    void testManagerSignals();
+    void testDeviceSignals();
+    void testDeviceExistence();
+    void testDeviceInterfaceIntrospection_data();
+    void testDeviceInterfaceIntrospection();
+    void testDeviceInterfaceIntrospectionCornerCases();
+    void testDeviceInterfaces();
+    void testInvalidPredicate();
+    void testPredicate();
+    void testQueryStorageVolumeOrProcessor();
+    void testQueryStorageVolumeOrStorageAccess();
+    void testQueryWithParentUdi();
+    void testListFromTypeProcessor();
+    void testListFromTypeInvalid();
+    void testSetupTeardown();
+    void testStorageAccessFromPath();
+    void testStorageAccessFromPath_data();
 
-#if defined(Q_OS_WIN)
-#include <qt_windows.h>
-#define setenv(x, y, z) SetEnvironmentVariableA(x, y)
-#endif
+private:
+    Solid::Backends::Fake::FakeManager *fakeManager;
+};
+
+QTEST_MAIN(SolidHwTest)
 
 void SolidHwTest::initTestCase()
 {
@@ -152,8 +174,8 @@ void SolidHwTest::testDeviceSignals()
     Solid::Device device("/org/kde/solid/fakehw/platform_floppy_0_storage_virt_volume");
 
     // We'll spy our floppy
-    connect(device.as<Solid::GenericInterface>(), SIGNAL(propertyChanged(QMap<QString, int>)), this, SLOT(slotPropertyChanged(QMap<QString, int>)));
     QSignalSpy condition_raised(device.as<Solid::GenericInterface>(), SIGNAL(conditionRaised(QString, QString)));
+    QSignalSpy property_changed(device.as<Solid::GenericInterface>(), SIGNAL(propertyChanged(QMap<QString, int>)));
 
     fake->setProperty("mountPoint", "/tmp.foo"); // The button is now pressed (modified property)
     fake->raiseCondition("Floppy Closed", "Why not?"); // Since it's a LID we notify this change
@@ -161,24 +183,24 @@ void SolidHwTest::testDeviceSignals()
     fake->removeProperty("hactar"); // We remove a property
 
     // 3 property changes occurred in the device
-    QCOMPARE(m_changesList.count(), 3);
+    QCOMPARE(property_changed.count(), 3);
 
     QMap<QString, int> changes;
 
     // First one is a "PropertyModified" for "button.state"
-    changes = m_changesList.at(0);
+    changes = property_changed.at(0).at(0).value<QMap<QString, int>>();
     QCOMPARE(changes.count(), 1);
     QVERIFY(changes.contains("mountPoint"));
-    QCOMPARE(changes["stateValue"], (int)Solid::GenericInterface::PropertyModified);
+    QCOMPARE(changes["mountPoint"], (int)Solid::GenericInterface::PropertyModified);
 
     // Second one is a "PropertyAdded" for "hactar"
-    changes = m_changesList.at(1);
+    changes = property_changed.at(1).at(0).value<QMap<QString, int>>();
     QCOMPARE(changes.count(), 1);
     QVERIFY(changes.contains("hactar"));
     QCOMPARE(changes["hactar"], (int)Solid::GenericInterface::PropertyAdded);
 
     // Third one is a "PropertyRemoved" for "hactar"
-    changes = m_changesList.at(2);
+    changes = property_changed.at(2).at(0).value<QMap<QString, int>>();
     QCOMPARE(changes.count(), 1);
     QVERIFY(changes.contains("hactar"));
     QCOMPARE(changes["hactar"], (int)Solid::GenericInterface::PropertyRemoved);
@@ -368,27 +390,30 @@ void SolidHwTest::testPredicate()
 void SolidHwTest::testQueryStorageVolumeOrProcessor()
 {
     auto list = Solid::Device::listFromQuery("[Processor.number==1 OR IS StorageVolume]");
-    QCOMPARE(list.size(), 10);
+    QCOMPARE(list.size(), 13);
 
     // make sure predicate case-insensitiveness is sane
     list = Solid::Device::listFromQuery("[Processor.number==1 or is StorageVolume]");
-    QCOMPARE(list.size(), 10);
+    QCOMPARE(list.size(), 13);
     list = Solid::Device::listFromQuery("[Processor.number==1 oR Is StorageVolume]");
-    QCOMPARE(list.size(), 10);
+    QCOMPARE(list.size(), 13);
     QStringList expected{"/org/kde/solid/fakehw/acpi_CPU1",
                          "/org/kde/solid/fakehw/platform_floppy_0_storage_virt_volume",
+                         "/org/kde/solid/fakehw/volume_0000_unmounted_storage",
                          "/org/kde/solid/fakehw/volume_label_SOLIDMAN_BEGINS",
                          "/org/kde/solid/fakehw/volume_part1_size_993284096",
                          "/org/kde/solid/fakehw/volume_part2_size_1024",
                          "/org/kde/solid/fakehw/volume_part5_size_1048576",
                          "/org/kde/solid/fakehw/volume_uuid_5011",
                          "/org/kde/solid/fakehw/volume_uuid_c0ffee",
+                         "/org/kde/solid/fakehw/volume_uuid_cleartext_data_0123",
+                         "/org/kde/solid/fakehw/volume_uuid_encrypted_0123",
                          "/org/kde/solid/fakehw/volume_uuid_f00ba7",
                          "/org/kde/solid/fakehw/volume_uuid_feedface"};
     QCOMPARE(to_string_list(list), expected);
 
     list = Solid::Device::listFromQuery("[IS Processor OR IS StorageVolume]");
-    QCOMPARE(list.size(), 11);
+    QCOMPARE(list.size(), 14);
     expected.prepend("/org/kde/solid/fakehw/acpi_CPU0");
     QCOMPARE(to_string_list(list), expected);
 }
@@ -467,9 +492,41 @@ void SolidHwTest::testSetupTeardown()
     QCOMPARE(args.at(0).toBool(), true);
 }
 
-void SolidHwTest::slotPropertyChanged(const QMap<QString, int> &changes)
+void SolidHwTest::testStorageAccessFromPath()
 {
-    m_changesList << changes;
+    QFETCH(QString, path);
+    QFETCH(QString, deviceUdi);
+
+    const auto prefix = QStringLiteral("/org/kde/solid/fakehw/");
+
+    const Solid::Device device = Solid::Device::storageAccessFromPath(path);
+#if defined(Q_OS_WIN)
+    // storageAccessFromPath has never been working correctly on Windows
+    // Fixing this is left for someone with access to a Windows machine
+    return;
+#endif
+
+    QVERIFY(device.isValid());
+    QCOMPARE(device.udi(), prefix + deviceUdi);
+
+    auto storage = device.as<Solid::StorageAccess>();
+    QVERIFY(storage);
+    QVERIFY(!storage->filePath().isEmpty());
+    QVERIFY(path.startsWith(storage->filePath()));
 }
 
-#include "moc_solidhwtest.cpp"
+void SolidHwTest::testStorageAccessFromPath_data()
+{
+    QTest::addColumn<QString>("path");
+    QTest::addColumn<QString>("deviceUdi");
+
+    QTest::addRow("Root")            << QStringLiteral("/")           << QStringLiteral("volume_uuid_feedface");
+    QTest::addRow("Mount directory") << QStringLiteral("/media")      << QStringLiteral("volume_uuid_feedface");
+    QTest::addRow("NFS mount")       << QStringLiteral("/media/nfs")  << QStringLiteral("fstab/thehost/solidpath");
+    QTest::addRow("Home directory")  << QStringLiteral("/home")       << QStringLiteral("volume_uuid_c0ffee");
+    QTest::addRow("Other home")      << QStringLiteral("/home_other") << QStringLiteral("volume_uuid_feedface");
+    QTest::addRow("User home")       << QStringLiteral("/home/user")  << QStringLiteral("volume_uuid_c0ffee");
+    QTest::addRow("LUKS")            << QStringLiteral("/data")       << QStringLiteral("volume_uuid_cleartext_data_0123");
+}
+
+#include "solidhwtest.moc"
